/*
 This source file is part of the Swift.org open source project

 Copyright (c) 2021 Apple Inc. and the Swift project authors
 Licensed under Apache License v2.0 with Runtime Library Exception

 See https://swift.org/LICENSE.txt for license information
 See https://swift.org/CONTRIBUTORS.txt for Swift project authors
*/

import Foundation

public enum IndexingError: DescribedError {
    /**
     A page or other piece of documentation doesn't have a title, so it can't
     have a useful search record.
     */
    case missingTitle(ResolvedTopicReference)
    
    public var errorDescription: String {
        switch self {
        case let .missingTitle(topicReference):
            return "\(topicReference.absoluteString.singleQuoted) has an empty title and so can't have a usable search result"
        }
    }
}
